import json

from django import forms
from django.contrib import admin
from django.shortcuts import render, get_object_or_404
from django.http import HttpResponse
from django.urls import path

from drip.models import Drip, SentDrip, QuerySetRule
from drip.drips import configured_message_classes, message_class_for
from drip.utils import get_user_model, get_simple_fields


class QuerySetRuleInline(admin.TabularInline):
    model = QuerySetRule


class DripForm(forms.ModelForm):
    message_class = forms.ChoiceField(
        choices=(
            (k, '{k} ({v})'.format(k=k, v=v))
            for k, v in configured_message_classes().items()
        ),
    )

    class Meta:
        model = Drip
        exclude = []


class DripAdmin(admin.ModelAdmin):
    list_display = ('name', 'enabled', 'message_class')
    inlines = [
        QuerySetRuleInline,
    ]
    form = DripForm
    users_fields = []

    def av(self, view):
        return self.admin_site.admin_view(view)

    def timeline(self, request, drip_id, into_past, into_future):
        """
        Return a list of people who should get emails.
        """

        drip = get_object_or_404(Drip, id=drip_id)

        shifted_drips = []
        seen_users = set()
        for shifted_drip in drip.drip.walk(
            into_past=int(into_past), into_future=int(into_future)+1
        ):
            shifted_drip.prune()
            shifted_drips.append(
                {
                    'drip': shifted_drip,
                    'qs': shifted_drip.get_queryset().exclude(
                        id__in=seen_users,
                    ),
                },
            )
            seen_users.update(
                shifted_drip.get_queryset().values_list('id', flat=True)
            )

        return render(request, 'drip/timeline.html', locals())

    def get_mime_html_from_alternatives(self, alternatives):
        html = ''
        mime = ''
        for body, mime in alternatives:
            if mime == 'text/html':
                html = body
                mime = 'text/html'
        return html, mime

    def get_mime_html(self, drip, user):
        drip_message = message_class_for(
            drip.message_class,
        )(drip.drip, user)
        if drip_message.message.alternatives:
            return self.get_mime_html_from_alternatives(
                drip_message.message.alternatives
            )
        html = drip_message.message.body
        mime = 'text/plain'
        return html, mime

    def view_drip_email(
        self, request, drip_id, into_past, into_future, user_id
    ):

        drip = get_object_or_404(Drip, id=drip_id)
        User = get_user_model()
        user = get_object_or_404(User, id=user_id)

        html, mime = self.get_mime_html(drip, user)

        return HttpResponse(html, content_type=mime)

    def build_extra_context(self, extra_context):
        extra_context = extra_context or {}
        User = get_user_model()
        if not self.users_fields:
            self.users_fields = json.dumps(get_simple_fields(User))
        extra_context['field_data'] = self.users_fields
        return extra_context

    def add_view(self, request, extra_context=None):
        return super(DripAdmin, self).add_view(
            request, extra_context=self.build_extra_context(extra_context),
        )

    def change_view(self, request, object_id, extra_context=None):
        return super(DripAdmin, self).change_view(
            request,
            object_id,
            extra_context=self.build_extra_context(extra_context)
        )

    def get_urls(self):
        urls = super(DripAdmin, self).get_urls()
        my_urls = [
            path(
                '<int:drip_id>/timeline/<int:into_past>/<int:into_future>/',
                self.av(self.timeline),
                name='drip_timeline'
            ),
            path(
                '<int:drip_id>/timeline/<int:into_past>/'
                '<int:into_future>/<int:user_id>/',
                self.av(self.view_drip_email),
                name='view_drip_email'
            )
        ]
        return my_urls + urls


admin.site.register(Drip, DripAdmin)


class SentDripAdmin(admin.ModelAdmin):
    list_display = [f.name for f in SentDrip._meta.fields]
    ordering = ['-id']


admin.site.register(SentDrip, SentDripAdmin)



from django.conf import settings
from apscheduler.schedulers.background import BackgroundScheduler

from django.core.management import call_command


DRIP_SCHEDULE_SETTINGS = getattr(
    settings, 'DRIP_SCHEDULE_SETTINGS', {}
)


DRIP_SCHEDULE = DRIP_SCHEDULE_SETTINGS.get(
    'DRIP_SCHEDULE', False
)
DRIP_SCHEDULE_DAY_OF_WEEK = DRIP_SCHEDULE_SETTINGS.get(
    'DRIP_SCHEDULE_DAY_OF_WEEK', 0
)
DRIP_SCHEDULE_HOUR = DRIP_SCHEDULE_SETTINGS.get(
    'DRIP_SCHEDULE_HOUR', 0
)
DRIP_SCHEDULE_MINUTE = DRIP_SCHEDULE_SETTINGS.get(
    'DRIP_SCHEDULE_MINUTE', 0
)


def cron_send_drips():
    def call_send_drips_command():
        call_command('send_drips')

    if DRIP_SCHEDULE:
        cron_scheduler = BackgroundScheduler()
        cron_scheduler.add_job(
            call_send_drips_command,
            'cron',
            day_of_week=DRIP_SCHEDULE_DAY_OF_WEEK,
            hour=DRIP_SCHEDULE_HOUR,
            minute=DRIP_SCHEDULE_MINUTE,
            timezone=getattr(settings, 'TIME_ZONE', 'UTC'),
        )
        cron_scheduler.start()



from django.test import TestCase
from django.core import mail
from django.conf import settings

from drip.models import Drip, QuerySetRule
from drip.drips import DripMessage
from drip.utils import get_user_model


# Used by CustomMessagesTest
class PlainDripEmail(DripMessage):
    @property
    def message(self):
        if not self._message:
            email = mail.EmailMessage(
                self.subject,
                self.plain,
                self.from_email,
                [self.user.email],
            )
            self._message = email
        return self._message


class CustomMessagesTest(TestCase):
    def setUp(self):
        self.User = get_user_model()

        self.old_msg_classes = getattr(settings, 'DRIP_MESSAGE_CLASSES', None)
        self.user = self.User.objects.create(
            username='customuser',
            email='custom@example.com',
        )
        self.model_drip = Drip.objects.create(
            name='A Custom Week Ago',
            subject_template='HELLO {{ user.username }}',
            body_html_template='<h2>This</h2> is an <b>example</b>'
            ' html <strong>body</strong>.',
        )
        QuerySetRule.objects.create(
            drip=self.model_drip,
            field_name='id',
            lookup_type='exact',
            field_value=self.user.id,
        )

    def tearDown(self):
        if self.old_msg_classes is None:
            if hasattr(settings, 'DRIP_MESSAGE_CLASSES'):
                del settings.DRIP_MESSAGE_CLASSES
        else:
            settings.DRIP_MESSAGE_CLASSES = self.old_msg_classes

    def test_default_email(self):
        result = self.model_drip.drip.send()
        self.assertEqual(1, result)
        self.assertEqual(1, len(mail.outbox))
        email = mail.outbox.pop()
        self.assertIsInstance(email, mail.EmailMultiAlternatives)

    def test_custom_added_not_used(self):
        settings.DRIP_MESSAGE_CLASSES = {'plain': 'drip.tests.PlainDripEmail'}
        result = self.model_drip.drip.send()
        self.assertEqual(1, result)
        self.assertEqual(1, len(mail.outbox))
        email = mail.outbox.pop()
        # Since we did not specify custom class, default should be used.
        self.assertIsInstance(email, mail.EmailMultiAlternatives)

    def test_custom_added_and_used(self):
        settings.DRIP_MESSAGE_CLASSES = {
            'plain': 'drip.tests.test_custom_messages.PlainDripEmail',
        }
        self.model_drip.message_class = 'plain'
        self.model_drip.save()
        result = self.model_drip.drip.send()
        self.assertEqual(1, result)
        self.assertEqual(1, len(mail.outbox))
        email = mail.outbox.pop()
        # In this case we did specify the custom key,
        # so message should be of custom type.
        self.assertIsInstance(email, mail.EmailMessage)

    def test_override_default(self):
        settings.DRIP_MESSAGE_CLASSES = {
            'default': 'drip.tests.test_custom_messages.PlainDripEmail',
        }
        result = self.model_drip.drip.send()
        self.assertEqual(1, result)
        self.assertEqual(1, len(mail.outbox))
        email = mail.outbox.pop()
        self.assertIsInstance(email, mail.EmailMessage)



# Generated by Django 3.0.7 on 2020-11-25 13:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('drip', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='querysetrule',
            name='rule_type',
            field=models.CharField(choices=[('or', 'Or'), ('and', 'And')], default='and', max_length=3),
        ),
    ]



from django.core.management.base import BaseCommand

from drip.models import Drip


class Command(BaseCommand):
    def handle(self, *args, **options):
        for drip in Drip.objects.filter(enabled=True):
            drip.drip.run()
